from typing import Optional

from openstack.exceptions import ResourceNotFound

from interface_openstack.interface.image.image_manager import ImageManager
from interface_openstack.interface.image.image import Image
from interface_openstack.implementation.utils.connection import OpenStackConnection


class OpenstackImageManager(ImageManager, OpenStackConnection):

    @staticmethod
    def os_image_to_rc_image(os_image) -> Image:
        return Image(
            os_image.id,
            os_image.name,
            os_image.disk_format
        )

    def upload_file(self, name, path, format) -> Image:
        os_image = self.openstack.image.create_image(
            name=name,
            filename=path,
            disk_format=format
        )

        return self.os_image_to_rc_image(os_image)

    def get(self, id) -> Optional[Image]:
        try:
            os_image = self.openstack.image.get_image(id)
        except ResourceNotFound:
            return None

        return self.os_image_to_rc_image(os_image)

    def download(self, id):
        return self.openstack.image.download_image(id)

    def delete(self, id) -> bool:
        try:
            self.openstack.image.delete_image(id)
        except ResourceNotFound:
            return False

        return True

    def list(self) -> []:
        images = []

        for os_image in self.openstack.image.images():
            images.append(self.os_image_to_rc_image(os_image))

        return images
